//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// dumprom.H
//
//		Console Win32 Application which uses the 1394 Class API
//		from TI to dump a 1394 device ROM.
//
//		Developed using MSVC 5.0.
//
//	Bill Alexander
//	10/13/98
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#pragma pack(1)

#ifndef _1394API_H_
#include "1394api.h"
#endif

// Default 1394 Device GUIDs
#define DEFAULT_DEVICE_GUID			0x00A0B800

// Host GUIDs
#define TI_LYNX_HOST_GUID1			0x08002850
#define TI_LYNX_HOST_GUID2			0x08002851
#ifdef _APP_MAIN_MODULE_
DWORD HostGuids[] =
{
	TI_LYNX_HOST_GUID1,
	TI_LYNX_HOST_GUID2
};
#endif

// Program Information Structure
typedef struct ProgInfoStruc
{
	// Verbosity Level Command Line Option
	int VerbosityLevel;

	// Stop before exit command line option
	BOOL StopBeforeExit;

	// Flag to indicate whether or not the 1394 class driver is loaded
	// and initialized
	BOOL DriverInitialized;

	// Specified offset option
	BOOL UseSpecifiedOffset;
	DWORD SpecifiedOffset;

	// Connection handle to host 1394 adapter
	cls1394HANDLE hHost;

	// Connection handle to 1394 device
	cls1394HANDLE hDev;

	// Device Id
	DWORD DeviceId;

} ProgInfoT, *PProgInfoT;

// MyPrintf VerbosityLevel settings
#define VL_NONE				0
#define VL_ALWAYS			0
#define VL_ERROR			0
#define VL_MAJOR_STEP		1
#define VL_DEBUG			2


// 1394 Core 1394 Addresses
LARGE_INTEGER STATE_CLEAR		=	{0xF0000000, 0x0000FFFF};
LARGE_INTEGER STATE_SET			=	{0xF0000004, 0x0000FFFF};
LARGE_INTEGER NODE_IDS			=	{0xF0000008, 0x0000FFFF};
LARGE_INTEGER RESET_START		=	{0xF000000C, 0x0000FFFF};
LARGE_INTEGER SPLIT_TIMEOUT_H	=	{0xF0000018, 0x0000FFFF};
LARGE_INTEGER SPLIT_TIMEOUT_L	=	{0xF000001C, 0x0000FFFF};
LARGE_INTEGER BUSY_TIMEOUT		=	{0xF0000210, 0x0000FFFF};
LARGE_INTEGER CONFIG_ROM		=	{0xF0000400, 0x0000FFFF};

// Function Prototypes
int main(int argc, char *argv[]);
BOOL ProcessCommandLine (int argc, char *argv[], PProgInfoT pi);
void Usage (PProgInfoT pi);
void MyPrintf (PProgInfoT pi, int vl, char *fmt);
void MyPrintf1 (PProgInfoT pi, int vl, char *fmt, DWORD p1);
void MyPrintf2 (PProgInfoT pi, int vl, char *fmt, DWORD p1, DWORD p2);
BOOL WriteBuffer (
			cls1394HANDLE hDev,
			LARGE_INTEGER liAddress,
			PQUADLET Buffer,
			DWORD BufferSize);
BOOL WriteQuadlet (
			cls1394HANDLE hDev,
			LARGE_INTEGER liAddress,
			QUADLET qValue);
BOOL WriteVerifyQuadlet (
			cls1394HANDLE hDev,
			LARGE_INTEGER liAddress,
			QUADLET qValue,
			DWORD to);
BOOL ReadQuadlet(cls1394HANDLE hDev, LARGE_INTEGER liAddress, QUADLET *qValue);
BOOL HostOpen (cls1394HANDLE *hHost);
BOOL HostReadNodeId (cls1394HANDLE hHost, QUADLET *qValue);
BOOL DeviceOpen (DWORD DeviceId, cls1394HANDLE *hDev);
BOOL DumpConfigRom (PProgInfoT pi);
LARGE_INTEGER LI_HOST_SWAP (LARGE_INTEGER li);
void HostSwapQuadlet (QUADLET qValue, PQUADLET dst);


