/***********************************************************************
/*   COPYRIGHT (C) 1996 TEXAS INSTRUMENTS INCORPORATED.
/*   ALL RIGHTS RESERVED. ONLY TO BE USED ON TEXAS INSTRUMENTS SILICON.
/***********************************************************************
/*
/*  Project SoftLynx 1394 Drivers and API set

                        Texas Instruments
                        Copyright  1994. All Rights Reserved.

                        LANGUAGE:     ANSI C
                        SUBSYSTEM:    1394 Drivers
                        FILE:         1394Type.h
                        AUTHOR:       Richard Solomon


                        OVERVIEW
                        ========
                        This file contains types used by the
                        TI SoftLynx 1394 Microsoft class library function:
/**/

#ifndef _1394TYPE_H_
#define _1394TYPE_H_


#ifndef _WINDOWS_   /* Define needed windows types here if on NON-WINDOW */
typedef unsigned int   BOOL;
typedef unsigned char  BYTE;       /* 8  IEEE 1212 defn */
typedef unsigned short WORD;       /* 16 bits */
typedef unsigned int   UINT;       /* 16 or 32 bits */
typedef unsigned long  ULONG;      /* 32 bits */
typedef unsigned long  *PULONG;
typedef unsigned long  HANDLE;     /* 32 bits */
typedef unsigned long  *PHANDLE;

typedef void far *LPVOID;

typedef struct {
    ULONG   LowPart;
    ULONG   HighPart;
  } LARGE_INTEGER, *PLARGE_INTEGER;

#endif /* ifndef _WINDOWS_ */

typedef unsigned short DOUBLET;    /* IEEE 1212 defn */
typedef unsigned long  QUADLET;    /* IEEE 1212 defn */
typedef QUADLET*       PQUADLET;

/**
typedef struct _OCTLET {
        QUADLET Low;
        QUADLET High;
} OCTLET, *POCTLET;
/**/
typedef struct _OFFSET_1394 {
        QUADLET Low;
        DOUBLET High;
} OFFSET_1394, *P1394_OFFSET;


#define BusInfoBlockQcnt  5

/* SUB-BLOCK CSR ROM header quad breakdown */
typedef struct {
     WORD   CRC;
     WORD   QuadLgt;
} CRCInfoQuad, *pCRCInfoQuad;

typedef struct {
   union {
     QUADLET UID_hi;
     struct {
       unsigned     chip_id_hi      :  8;
       unsigned     node_vendor_id  : 24;
     }; /*struct*/
   }; /*union*/
   QUADLET chip_id_lo;
} WWUniqueID, *pWWUniqueID;

typedef union {
   QUADLET  Quads[BusInfoBlockQcnt];
   struct{
     unsigned Rom_CRC    : 16;
     unsigned CRC_Lgt    : 8;
     unsigned InfoLgt    : 8;
     QUADLET  BusName;
     struct {
        unsigned rsvd12      : 12;
        unsigned max_rec     :  4;
        unsigned cyc_clk_acc :  8;
        unsigned rsvd04      :  4;
        unsigned bmc         :  1;
        unsigned isc         :  1;
        unsigned cmc         :  1;
        unsigned irmc        :  1;
     };  /* struct */
     WWUniqueID   WWUID;
   }; /* struct */
} BusInfoBlock, *pBusInfoBlock;


/*---------------------------------------------------------------------------/
/* ASYNC Packet formats for:
/*     Write Response                 (6.2.2.1.2),
/*     Read Response for data quadlet (6.2.2.2.4),
/*     Read Response for data block   (6.2.2.3.3),
/*     Lock Response                  (6.2.2.3.4).
/*     R/W  Request for Quad/Block/Lock
/*
/*--------------------------------------------------------------------------*/
typedef struct {
    struct {     /* first quadlet */
        unsigned    pri     :4;
        unsigned    tcode   :4;
        unsigned    rt      :2;
        unsigned    tl      :6;
        unsigned    destination_id  :16;
    };
    union {      /* second quadlet */
      unsigned  dest_offset_hi  :16;
      struct {
        unsigned    reserved_1  :12;
        unsigned    rcode       : 4;
        unsigned    source_id   :16;
      }; /* struct */
    }; /* union 2nd Quad */

    union {      /* third quadlet */
      QUADLET     reserved_2;
      QUADLET     dest_offset_lo;
    }; /* union 3rd Quad */

    union {      /* fourth quadlet */
      QUADLET     quad_data;
      struct {
        unsigned    extended_tcode  :16;
        unsigned    data_length     :16;
      }; /* struct */
    }; /* union 4th Quad */

    QUADLET     block_data[100];      /* fifth quadlet */

} PACKET_HEADER, *pPACKET_HEADER;


#endif /* _1394TYPE_H_ */
