/***********************************************************************
/*   COPYRIGHT (C) 1996 TEXAS INSTRUMENTS INCORPORATED.
/*   ALL RIGHTS RESERVED. ONLY TO BE USED ON TEXAS INSTRUMENTS SILICON.
/***********************************************************************
/*
/*  Project SoftLynx 1394 Drivers and API set

                        Texas Instruments
                        Copyright  1994. All Rights Reserved.

                        LANGUAGE:       ANSI C
                        SUBSYSTEM:      1394 Drivers
                        FILE:           1394API.h
                        AUTHOR:         Dale Burger
                        CREATION DATE:  07 MAY 95


                        OVERVIEW
                        ========
                        This file contains definitions required for the
                        TI SoftLynx 1394 Microsoft class library as the
                        interface to an Applications Program

/* REVISION HISTORY:
/*
/*  STR #   INIT   DATE            SHORT TITLE
/* [xxxxx]  nnn  dd mmm yy ..............................................
/**/

#ifndef _1394API_H_
#define _1394API_H_


#include <windows.h>  /*DeviceIoControl, Types*/


#include "1394type.h"  /*Defined types for 1394 API*/

/*#define DllImport     __declspec( dllimport )  /*MSC defn*/
#define DllExport     __declspec( dllexport )  /*MSC defn*/

#ifdef  _CLS1394_GLOBALS
   #define _EXTERN
#else
   #define _EXTERN extern
#endif

/* DeviceIoControl Function Requests */
#define CLS_REQUEST_ALLOCATE_ADDRESS_RANGE       1
#define CLS_REQUEST_FREE_ADDRESS_RANGE           2
#define CLS_REQUEST_ASYNC_READ                   3
#define CLS_REQUEST_ASYNC_WRITE                  4
#define CLS_REQUEST_ASYNC_LOCK                   5
#define CLS_REQUEST_ALLOCATE_BANDWIDTH           6
#define CLS_REQUEST_ALLOCATE_CHANNEL             7
#define CLS_REQUEST_ALLOCATE_RESOURCES           8
#define CLS_REQUEST_ATTACH_BUFFERS               9
#define CLS_REQUEST_DETACH_BUFFERS              10
#define CLS_REQUEST_FREE_BANDWIDTH              11
#define CLS_REQUEST_FREE_CHANNEL                12
#define CLS_REQUEST_FREE_RESOURCES              13
#define CLS_REQUEST_ISOCH_LISTEN                14
#define CLS_REQUEST_ISOCH_QUERY_CYCLE_NUMBER    15
#define CLS_REQUEST_ISOCH_QUERY_RESOURCES       16
/** #define CLS_REQUEST_ISOCH_SET_BANDWIDTH         17  */
#define CLS_REQUEST_ISOCH_STOP                  18
#define CLS_REQUEST_ISOCH_TALK                  19
#define CLS_REQUEST_APPEND_UNIT_DIRECTORY_ENTRY 20
#define CLS_REQUEST_DELETE_UNIT_DIRECTORY_ENTRY 21
#define CLS_REQUEST_GET_LOCAL_HOST_INFO         22
#define CLS_REQUEST_GET_1394_ADDRESS            23
#define CLS_REQUEST_CONTROL                     24
#define CLS_REQUEST_GET_MAX_SPEED_FOR_NODES     25
#define CLS_REQUEST_SET_DEVICE_SPEED            26
#define CLS_REQUEST_GET_CONFIG_INFO             27
/*#define CLS_REQUEST_       X */

/* DeviceIoControl Error Return Codes.  Use GetLastError */
/* ACK codes (00-0F) see 1394 spec (6.2.5.2.2)           */
/* LLC codes (10-1F) see PCILynx spec PCL status section */
#define  CLASS1394_ACK_00                   0  /* rsvd */
#define  CLASS1394_ACK_COMPLETE             1
#define  CLASS1394_ACK_PENDING              2
#define  CLASS1394_ACK_03                   3  /* rsvd */
#define  CLASS1394_ACK_BUSY_X               4
#define  CLASS1394_ACK_BUSY_A               5
#define  CLASS1394_ACK_BUSY_B               6
#define  CLASS1394_ACK_07                   7  /* rsvd */
#define  CLASS1394_ACK_08                   8  /* rsvd */
#define  CLASS1394_ACK_09                   9  /* rsvd */
#define  CLASS1394_ACK_0A                0x0A  /* rsvd */
#define  CLASS1394_ACK_0B                0x0B  /* rsvd */
#define  CLASS1394_ACK_0C                0x0C  /* rsvd */
#define  CLASS1394_ACK_DATA_ERROR        0x0D  /*hardware error, data unavailable */
#define  CLASS1394_ACK_TYPE_ERROR        0x0E  /*incorrect request packet */
#define  CLASS1394_ACK_0F                0x0F  /* rsvd */
#define  CLASS1394_LLC_RETRYOVERRUN      0x10
#define  CLASS1394_LLC_TIMEOUT           0x11
#define  CLASS1394_LLC_FIFOUNDERRUN      0x12
#define  CLASS1394_LLC_13                0x13
#define  CLASS1394_LLC_14                0x14
#define  CLASS1394_LLC_NO_PKT_END        0x15
#define  CLASS1394_LLC_PIPELINE_ERR      0x16
#define  CLASS1394_LLC_17                0x17
#define  CLASS1394_LLC_18                0x18
#define  CLASS1394_LLC_19                0x19
#define  CLASS1394_LLC_1A                0x1A
#define  CLASS1394_LLC_1B                0x1B
#define  CLASS1394_LLC_1C                0x1C
#define  CLASS1394_LLC_1D                0x1D
#define  CLASS1394_LLC_CORRUPT_HEADER    0x1E
#define  CLASS1394_LLC_1F                0x1F
#define  CLASS1394_SUCCESS               0x20  /* Class Operation successful code */
#define  CLASS1394_GENERIC_FAILURE       0x21  /* Generic Fail code */
#define  CLASS1394_INVALID_REQUEST       0x22  /* DeviceIoControl */
#define  CLASS1394_WWUID_INVALID         0x23  /* CreateFile  */
#define  CLASS1394_WWUID_NOTFOUND        0x24  /* CreateFile  */
#define  CLASS1394_HANDLE_INVALID        0x25  /* CloseHandle */
#define  CLASS1394_HANDLE_NOTOPEN        0x26  /* CloseHandle */
#define  CLASS1394_SPEEDMAP_ERROR        0x27  /* Error accessing speed map */
#define  CLASS1394_NUM_DESTINATIONS_ERR  0x28  /* Num of destinations for speed < 0 */
#define  CLASS1394_RESPONSE_UNEXPECTED   0x29  /* rsvd rCode in response packet */
#define  CLASS1394_RESPONSE_CONFLICT     0x2A  /* resource conflict, retry request */
#define  CLASS1394_RESPONSE_DATAERR      0x2B  /* HW error, data unavailable */
#define  CLASS1394_RESPONSE_TYPEERR      0x2C  /* Unsupported/invalid field, invalid request */
#define  CLASS1394_RESPONSE_ADDRERR      0x2D  /* address not accessible  */
#define  CLASS1394_RESPONSE_ZERO_DATA    0x2E  /* zero data in data payload packet */
#define  CLASS1394_RESPONSE_TIMEOUT      0x2F  /* Timeout - no response in allotted time */
#define  CLASS1394_ISOALLOCRES_MEM_FAIL  0x30  /* Failed to allocate Internal Class Resource Struct */
#define  CLASS1394_SPEED_NOT_AVAIL       0x31  /* Failed to Acquire speed requested by user */
#define  CLASS1394_INVALID_ADDR_RNG      0x32  /* Failed to Allocate/Free 1394 Address Range */

/*enum cls1394Speed  { S100, S200, S400, SpeedFastest };*/
#define Speed100     0
#define Speed200     1
#define Speed400     2
#define SpeedFastest 3

/* Defines needed for CONTROL class calls */

#define CLASS1394_CONTROL_BUS_RESET      1
#define CLASS1394_CONTROL_SOFTWARE_RESET 2

/* Defines needed for ISO class calls */
#define FLAG_SYNCHRONIZE     0x1
#define FLAG_STRIP_HEADER    0x2

#define ResourceUsedInTalking   0                /* Must stay 0 - as defined in HAL - TRANSMIT */
#define ResourceUsedInListening 1                /* Must stay 1 - as defined in HAL - RECEIVE  */

// Access Type Definition
#define AccessTypeRead   0x1
#define AccessTypeWrite  0x2
#define AccessTypeLock   0x4

// Notify Type Definition
#define NotifyAfterRead  0x1
#define NotifyAfterWrite 0x2
#define NotifyAfterLock  0x4

/* Extended TCodes for Lock Transactions */
#define ETC_MASK_SWAP   0x0001
#define ETC_COMPARE_SWAP 0x0002
#define ETC_FETCH_ADD   0x0003
#define ETC_LITTLE_ADD  0x0004
#define ETC_BOUNDED_ADD 0x0005
#define ETC_WRAP_ADD    0x0006

/*--------------------------------------------------------------------------*/
/*                           Type Declarations                              */
/*--------------------------------------------------------------------------*/

typedef unsigned short P1394_NODE_ADDRESS;
typedef unsigned short *PP1394_NODE_ADDRESS;

enum cls1394Status { STATUS_FAIL, STATUS_SUCCESS, STATUS_IS_PENDING };

typedef unsigned long  cls1394HANDLE;  /*Handle defn for 1394 device object*/

typedef struct {
        QUADLET         *lpBuffer;
        ULONG           nLength;
        ULONG           fulAccessType;
        ULONG           fulNotificationOptions;
        LPVOID          lpCallback;
        LPVOID          lpContext;
        P1394_OFFSET    Required1394Offset;
        PULONG          lpAddressReturned;
        PLARGE_INTEGER  lp1394Address;
} clsAllocateAddressRange, *pclsAllocateAddressRange;

typedef struct {
        ULONG           nAddressesToFree;
        PLARGE_INTEGER  lp1394Address;
} clsFreeAddressRange, *pclsFreeAddressRange;

typedef struct {
      PLARGE_INTEGER   DestinationAddress;
      ULONG            nNumberOfBytesToRead;   /* from the remote node*/
      ULONG            nBlockSize;             /* if zero, use max packet size*/
      ULONG            fulFlags;               /* LoWord specifies any unique operation*/
                                               /* HiWord = 0 - Default Response Wait time (1 sec)*/
                                               /* HiWord > 0 - Max Response Wait time in milli-secs*/
      LPVOID           lpBuffer;               /* Points to a MDL which describes the device*/
                                               /* driver's buffer to which data will be */
                                               /* recived from the remote node. */
} clsAsyncRead, *pclsAsyncRead;

typedef struct {
      PLARGE_INTEGER   DestinationAddress;
      ULONG            nNumberOfBytesToWrite;  /* to the remote node*/
      ULONG            nBlockSize;             /* if zero, use max packet size*/
      ULONG            fulFlags;               /* LoWord specifies any unique operation*/
                                               /* HiWord = 0 - Default Response Wait time (1 sec)*/
                                               /* HiWord > 0 - Max Response Wait time in milli-secs*/
      LPVOID           lpBuffer;               /* Points to a MDL which describes the device*/
                                               /* driver's buffer to which data will be */
                                               /* recived from the remote node. */
} clsAsyncWrite, *pclsAsyncWrite;

typedef struct {
      PLARGE_INTEGER   DestinationAddress;
      ULONG            nNumberOfArgBytes;
      ULONG            nNumberOfDataBytes;
      ULONG            fulTransactionType;
      ULONG            fulFlags;
      ULONG            Arguments[2];
      ULONG            DataValues[2];
      LPVOID           lpBuffer;
} clsAsyncLock, *pclsAsyncLock;

typedef struct _ISOCH_DESCRIPTOR {
        struct _ISOCH_DESCRIPTOR  *Next;
        ULONG                      fulFlags;
        PVOID                      lpBuffer;
        ULONG              ulLength;
        ULONG                      ulSynchronize;
        ULONG              ulCycle;
        LARGE_INTEGER      SystemTime;
        LPVOID              lpCallback;
        LPVOID              lpWaterLineCallback;   // new field added
        DWORD              ulWaterLine;           // new field added
        LPVOID              lpContext;
        ULONG              Status;
        ULONG              PacketSize;            // new field added
        ULONG              ulReserved[4];
} ISOCH_DESCRIPTOR, *PISOCH_DESCRIPTOR;


/* Allocated Isochronous bandwidth.*/
typedef struct {
      ULONG      nMaxBytesPerFrameRequested;
      ULONG      fulSpeed;                    /* Can speed be selected??                    */
      PHANDLE    lpBandwidth;                 /* points to field which contain the returned */
                                              /* bandwidth handle to be used in releasing   */
                                              /* bandwidth resources at some later time.    */
      PULONG     lpBytesPerFrameAvailable;    /* Points to field which will contain the     */
                                              /* bytes per frame that's available after the */
                                              /* allocation succeeded or failed.            */
      PULONG     lpSpeedSelected;             /* Points to the speed that was selected in   */
                                              /* allocating bandwidth.                      */
} clsIsochAllocateBandwidth, *pclsIsochAllocateBandwidth;

/* Allocated Isochronous channnel number.*/
typedef struct {
      ULONG           nRequestedChannel;  /* should be -1 or any channel                  */
      PULONG          lpChannel;          /* Points to the field which will contain the   */
                                          /* returned channel if the allocation happened. */
      PLARGE_INTEGER  lpChannelAvailable; /* Points to the field which will contain a bit */
                                          /* mask of the available isochronous channels   */
                                          /* after the allocation succeeded or failed.    */
} clsIsochAllocateChannel, *pclsIsochAllocateChannel;

/* Free previously allocated Isochronous bandwidth.*/
typedef struct {
      HANDLE  hBandwidth;
} clsIsochFreeBandwidth, *pclsIsochFreeBandwidth;

/* Free previously allocated Isochronous channnel number.*/
typedef struct {
      ULONG   nChannel;
} clsIsochFreeChannel, *pclsIsochFreeChannel;

/* Set a Bandwidth handler's bandwidth requirement.                        */
/* This is useful if a bandwidth must be released but the bandwidth has an */
/* incorrect or unknown bandwidth setting.                                 */
typedef struct {
      HANDLE          hBandwidth;
      ULONG           nMaxBytesPerFrame;
} clsIsochSetBandwidth, *pclsIsochSetBandwidth;

typedef struct {
    ULONG           nChannel;
    HANDLE          hResources;
    ULONG           fulFlags;
    ULONG           nStartCycle;
    LARGE_INTEGER   StartTime;
    ULONG           ulSynchronize;
    ULONG           ulTag;
}clsIsochListen, *pclsIsochListen;

typedef struct {
    ULONG           nChannel;
    HANDLE          hResources;
    ULONG           fulFlags;
    ULONG           nStopCycle;
    LARGE_INTEGER   StopTime;
    ULONG           ulSynchronize;
    ULONG           ulTag;
}clsIsochStop, *pclsIsochStop;

typedef struct {
    ULONG           nChannel;
    HANDLE          hResources;
    ULONG           fulFlags;
    ULONG           nStartCycle;
    LARGE_INTEGER   StartTime;
    ULONG           ulSynchronize;
    ULONG           ulTag;
}clsIsochTalk, *pclsIsochTalk;

typedef struct {
    PULONG           lpBytesPerFrameAvailable;
    PLARGE_INTEGER   lpChannelsAvailable;
}clsIsochQueryResources, *pclsIsochQueryResources;

typedef struct {
    HANDLE            hResources;
        DWORD             Channel;                   /* Added for Sai */
    PISOCH_DESCRIPTOR lpIsochDescriptor;
} clsIsochAttachBuffers, *pclsIsochAttachBuffers;

typedef struct {
    ULONG    fulSpeed;
    ULONG    fulFlags;
    PHANDLE  lpResources;
} clsIsochAllocateResources, *pclsIsochAllocateResources;

typedef struct {
    HANDLE            hResources;
    PISOCH_DESCRIPTOR lpIsochDescriptor;
} clsIsochDetachBuffers, *pclsIsochDetachBuffers;

typedef struct {
    HANDLE  hResources;
} clsIsochFreeResources, *pclsIsochFreeResources;

/*Capability flags for INFO_BLOCK1 structure*/
#define BUSCAPS_USES_PIO                        1
#define BUSCAPS_USES_DMA                        2
#define BUSCAPS_USES_BUSMASTERING               4
#define BUSCAPS_ALWAYS_RETURNS_1_1394_ADDR      8
#define BUSCAPS_USES_SCATTER_GATHER_FOR_ASYNC  16
#define BUSCAPS_BACKPLANE_ENVIRONMENT          32
#define BUSCAPS_ALLOWS_SYNC_ON_ISOCH_SY        64
#define BUSCAPS_ALLOWS_SYNC_ON_ISOCH_TAG      128

typedef struct  {
    ULONG  fulCapabilities;
        ULONG  nClassDriverVersion;
        ULONG  nPortDriverVersion;
        ULONG  nDMAResourcesAvail;
} INFO_BLOCK1, *PINFO_BLOCK1;

typedef struct {
      ULONG         nlevel;
      PINFO_BLOCK1  pInfoBlock1;
} clsGetLocalHostInformation, *pclsGetLocalHostInformation;


typedef struct {
         PP1394_NODE_ADDRESS   lpNodeAddress;
}clsGet1394AddressFromDeviceObject, *pclsGet1394AddressFromDeviceObject;


typedef struct {
        ULONG  ulloControlCode;
        LPVOID lpInBuffer;
        ULONG  ulInBufferLength;
        LPVOID lpOutBuffer;
        ULONG  ulOutBufferLength;
    ULONG  ulBytesReturned;
} clsControl, *pclsControl;

typedef struct {
         ULONG           ulNumberOfDestinations;
         cls1394HANDLE   hDestinationDeviceObjects[64];
         PULONG          lpSpeed;
}clsGetMaxSpeedBetweenDevices, *pclsGetMaxSpeedBetweenDevices;

typedef struct {
    PULONG  lpCycleNumber;
} IsochQueryCurrentCycleNumber, *pIsochQueryCurrentCycleNumber;

typedef struct {
         ULONG        fulSpeed;
}clsSetDeviceSpeed, *pclsSetDeviceSpeed;

typedef char* VendorName;
typedef char* ModelName;

typedef struct{
      BOOL    Active;
      WORD    DeviceEntry;
      WORD    NodeID;
      ULONG   VendID_DevType;
      ULONG   SerialNo;
      VendorName Vendor;
      ModelName  Model;
} DeviceInfo;

/*Prototype Declarations*/

/*--------------------------------------------------------------------------*/
/*                           Exported Function Declarations                 */
/*--------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif  /*ifdef __cplusplus*/

/************ Class 1394 API functions */

BOOL GetDeviceInfo( WORD  DeviceEntry,    /* the nth device entry 0-Host */
                    DeviceInfo* sDevInfo );

BOOL cls1394Initialize( BOOL* BusResetFlag );

BOOL cls1394Terminate();

BOOL  cls1394CloseHandle( cls1394HANDLE hHnd );

cls1394HANDLE cls1394CreateFile
      (  ULONG VendorID_DeviceType, /* the IEEE Vendor ID and vendor device type to open */
             WORD DeviceEntry );        /* the nth entry for this type device */
/*
/*  Special Provisions:
/*
/*    Open "HOST"  use valid VendorID_DeviceType, DeviceEntry = 0
/*
/*    Open Non-Compliant Device (ie not respond to query).
/*
/*        VendorID_DeviceType = 0,  DeviceEntry = nth desired
/*
/*        NOTE:  Device handles for NC devices will be invalid after
/*               a bus reset.  These devices must be reopened.
/**/

BOOL cls1394DeviceIoControl
      (
          cls1394HANDLE  hDevice,       /* handle to device of interest                     */
          DWORD  dwIoControlCode,       /* control code of operation to perform             */
          LPVOID  lpInBuffer,           /* pointer to buffer to supply input data           */
          DWORD  nInBufferSize,         /* size of input buffer                             */
          LPVOID  lpOutBuffer,          /* pointer to buffer to receive output data         */
          DWORD  nOutBufferSize,        /* size of output buffer                            */
          LPDWORD  lpBytesReturned,     /* ptr to variable to receive output byte count     */
          LPOVERLAPPED  lpOverlapped    /* ptr to overlapped structure for asynch operation */
      );

ULONG cls1394GetLastError();

PVOID GetAdapterAddress(); /* Valid ONLY after cls1394Initialize */

/******** Bus Manager Supplied Functions *************/

/* This function returns the Bus Manager's NODE ID. (6 bits) */
DWORD GetBusManagerID();

BOOL bReadPhyReg(DWORD dwReadPhy, DWORD *pdwParam);

/* Compute the 16 bit CRC over n QUADLETS, Either per 1994 or 1991 spec */
WORD CrcCalculate(DWORD* pQuadData, int nNumQuads, BOOL CRC1991 );

void ctDelay(ULONG N_x_125usecs);    /* # of 125 uSec intervals to wait */
void ctMillisec( ULONG Millisecs );  /* # of   1 mSec intervals to wait */
void ctMicrosec( ULONG Microsecs );  /* # of   1 uSec intervals to wait */

/*--------------------------------------------------------------------------*/
/*                           Variable Declarations                          */
/*--------------------------------------------------------------------------*/


#ifdef __cplusplus
}
#endif  /*ifdef __cplusplus*/

#endif  /*_1394API_H_*/
