/*--------------------------------------------------------------------------*/
/* Filename:    snapshot.c                                                  */
/* Description: Win-32 Console Application for Windows 95 that captures a   */
/*              video frame from a 1394 digital camera (Sony CCM-DS250)     */
/*              connected to a 1394 PCI Digital Data Port(tm) based on the  */
/*              Texas Instruments PCI Lynx(tm) TSB12LV21 chip and converts  */
/*              the image from YUV 4:2:2 to 24 bit RGB and writes it to a   */
/*              DIB file named IMAGE.BMP.                                   */
/*                                                                          */
/*              This file must be linked with the cls1394.lib file and      */
/*              used in conjunction with the cls1394.dll both provided      */
/*              by Texas Instruments and is part of their WIndows 95 API    */
/*              software available on TI's website: http://www.ti.com       */              
/*                                                                          */
/* http://www.seqadvtech.com                                                */
/* email: info@seqadvtech.com                                               */
/*--------------------------------------------------------------------------*/
/* (C) Copyright 1996-1997 by Sequoia Advanced Technologies, Inc.           */
/* All Rights Reserved.                                                     */
/*--------------------------------------------------------------------------*/
/* Digital Data Port is a trademark of Sequoia Advanced Technologies, Inc.  */
/* PCILynx is a trademark of Texas Instruments Corporation                  */
/*--------------------------------------------------------------------------*/
#define STRICT
#include <stdlib.h>
#include <stdio.h>
#include <windows.h>

//
// The 1394api.h header file is provided by Texas Instruments to be
// with their Windows 95 API software.  The software is available on
// TI's website: http://www.ti.com
//
#include "..\inc\1394api.h"


#include "snapshot.h"

main(int argc, char *argv[])
{
    cls1394HANDLE hDev;
    PBYTE pbyDibBuffer;
    DWORD dwBytesRead;
    HANDLE hFile;
	BOOL BusReset;
    
    printf("Digtal Camera Frame Capture Utility\n");
    printf("Copyright 1997 by Sequoia Advanced Technologies, Inc.\n");
    printf("All Rights Reserved\n\n");
    printf("File will be saved in IMAGE.BMP.\n\n");


//    printf("cls1394Initialize %08X\n", GetTickCount());
	BusReset = TRUE;
    cls1394Initialize(&BusReset);
    
    //
    // fetch handle to Sony CCM-DS250 digital camera
    //

//    printf("cls1394CreateFile %08X\n", GetTickCount());
    hDev = cls1394CreateFile(0x08004602,1);

    if(!hDev)
    {
        printf("Error: Camera Not Found...\n\n");
        cls1394Terminate();
        exit(0);
    }

//    printf("AutoWhiteBalance %08X\n", GetTickCount());
    AutoWhiteBalance(hDev);

    //
    //  let white balance stabilize
    //
    Sleep(100);                 

    pbyDibBuffer = SnapShot(hDev);

//    printf("CreateFile %08X\n", GetTickCount());
    hFile = CreateFile("IMAGE.BMP",GENERIC_READ | GENERIC_WRITE, 0,
                        NULL,OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL,NULL);

//    printf("WriteFile %08X\n", GetTickCount());
    WriteFile(hFile, pbyDibBuffer, ((320*240*3)+54), &dwBytesRead, NULL);

//    printf("CloseHandle %08X\n", GetTickCount());
    CloseHandle(hFile);

//    printf("cls1394CloseHandle %08X\n", GetTickCount());
    cls1394CloseHandle(hDev);

//    printf("cls1394Terminate %08X\n", GetTickCount());
    cls1394Terminate();

    printf("Image Captured...That's all folks!\n\n");
	return (0);
}

/*==========================================================================*/
/*          Routine: SnapShot                                               */
/*		                                                                    */
/*      Description: Returns a Device Independent Bitmap from the           */
/*                   Sony 1394 camera                                       */
/*                                                                          */
/*==========================================================================*/
PBYTE SnapShot(cls1394HANDLE hDev)
{
    DWORD dwChannel, dwSpeed, dwBytesPerFrame, i, j;
    HANDLE hBandwidth, hResource;
    LARGE_INTEGER liAddress;
    QUADLET qValue;
    ISOCH_DESCRIPTOR id;
    PBYTE pbyDibData;
    
//    printf("AllocateChannel %08X\n", GetTickCount());
    AllocateChannel(hDev,&dwChannel);

    dwSpeed = Speed100;
    dwBytesPerFrame = 640;
    
//    printf("AllocateBandwidth %08X\n", GetTickCount());
    AllocateBandwidth(hDev,&dwSpeed,&dwBytesPerFrame,&hBandwidth);

//    printf("AllocateResources %08X\n", GetTickCount());
    AllocateResources(hDev, Speed100, ResourceUsedInListening, &hResource);

    id.Next = NULL;
    id.fulFlags = (FLAG_SYNCHRONIZE + FLAG_STRIP_HEADER);
    id.lpBuffer = (PVOID) malloc(320*240*2);
    id.ulLength = (320*240*2);
    id.ulSynchronize = 1;
    id.ulCycle = 0;
    id.SystemTime.LowPart = 0;
    id.SystemTime.HighPart = 0;
    id.lpCallback = NULL;
    id.lpWaterLineCallback = NULL;
    id.ulWaterLine = 0;
    id.lpContext = 0;
    id.Status = 0;
    id.ulReserved[0] = 0;    
    id.ulReserved[1] = 0;    
    id.ulReserved[2] = 0;    
    id.ulReserved[3] = 0;    
    id.PacketSize = 640;

//    printf("AttachBuffers %08X\n", GetTickCount());
    AttachBuffers(hDev, hResource, dwChannel, &id);

//    printf("IsochListen %08X\n", GetTickCount());
    IsochListen(hDev, hResource, dwChannel);

    //
    // Set up the control registers on the CCM-DS250
    //

//    printf("WriteQuadlet %08X\n", GetTickCount());
    liAddress.HighPart = 0xFFFF;
    liAddress.LowPart = 0x0F0F00610;        //Set Camera Power
    qValue = 0x80000000;
    WriteQuadlet(hDev, liAddress, qValue);    

//    printf("WriteQuadlet %08X\n", GetTickCount());
    liAddress.LowPart = 0x0F0F00600;        //Set Frame Rate
    qValue = 0x80000000;
    WriteQuadlet(hDev, liAddress, qValue);

//    printf("WriteQuadlet %08X\n", GetTickCount());
    liAddress.LowPart = 0xF0F00604;         //Set Video Mode
    qValue = 0x20000000;
    WriteQuadlet(hDev, liAddress, qValue);

//    printf("WriteQuadlet %08X\n", GetTickCount());
    liAddress.LowPart = 0xF0F00608;         //Set Video Format
    qValue = 0;
    WriteQuadlet(hDev, liAddress, qValue);

//    printf("WriteQuadlet %08X\n", GetTickCount());
    liAddress.LowPart = 0xF0F0060C;         //Set ISO Channel - ISO Speed
    qValue = 0x00000000;
    WriteQuadlet(hDev, liAddress, qValue);

//    printf("WriteQuadlet %08X\n", GetTickCount());
    liAddress.LowPart = 0xF0F00614;         //Set ISO Enable
    qValue = 0x80000000;
    WriteQuadlet(hDev, liAddress, qValue);

    //
    //  This delay is designed to allow the isochronous buffer to fill
    //  with the video frame. It is an arbitrary time value that we 
    //  chose.
    //

    i = GetTickCount();
    i += 100;
    while(1)
    {
        j = GetTickCount();
        if(j >= i)
        {
            break;
        }                
    }

//    printf("WriteQuadlet %08X\n", GetTickCount());
    liAddress.LowPart = 0xF0F00614;
    qValue = 0;
    WriteQuadlet(hDev, liAddress, qValue);
    
//    printf("IsochStop %08X\n", GetTickCount());
    IsochStop(hDev, hResource, dwChannel);

//    printf("DetachBuffers %08X\n", GetTickCount());
    DetachBuffers(hDev, hResource, &id);

//    printf("FreeResources %08X\n", GetTickCount());
    FreeResources(hDev, hResource);

//    printf("FreeBandwidth %08X\n", GetTickCount());
    FreeBandwidth(hDev,hBandwidth);

//    printf("FreeChannel %08X\n", GetTickCount());
    FreeChannel(hDev,dwChannel);

//    printf("Yuv2BID %08X\n", GetTickCount());
    pbyDibData = Yuv2DIB(id.lpBuffer);

//    printf("free %08X\n", GetTickCount());
    free(id.lpBuffer);

    return(pbyDibData);            
}

/*==========================================================================*/
/*          Routine: AutoWhiteBalance                                       */
/*		                                                                    */
/*      Description: Tells the camera to automatically adjust its white     */
/*                   balance.                                               */
/*                                                                          */
/*==========================================================================*/
BOOL AutoWhiteBalance(cls1394HANDLE hDev)
{
    LARGE_INTEGER liAddress;
    liAddress.HighPart = 0xFFFF;
    liAddress.LowPart = 0x0F0F0080C;        
    return(WriteQuadlet(hDev, liAddress, 0x83000000));
}

/*==========================================================================*/
/*          Routine: AllocateChannel                                        */
/*		                                                                    */
/*      Description: Allocates an isochronous channel for subsequent use    */
/*                                                                          */
/*==========================================================================*/
BOOL AllocateChannel(cls1394HANDLE hDev, PULONG lpChannel)
{
    clsIsochAllocateChannel ac;
    LARGE_INTEGER ChannelsAvailable;

    ac.nRequestedChannel = 0xFFFFFFFF;
    ac.lpChannel = lpChannel;
    ac.lpChannelAvailable = &ChannelsAvailable;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_ALLOCATE_CHANNEL,
                                  &ac,
                                  sizeof(clsIsochAllocateChannel),
                                  0,
                                  0,
                                  0,
                                  0));
}

/*==========================================================================*/
/*          Routine: AllocateBandwidth                                      */
/*		                                                                    */
/*      Description: Allocates isochronous bandwidth for subsequent use     */
/*                                                                          */
/*==========================================================================*/
BOOL AllocateBandwidth(cls1394HANDLE hDev, PULONG lpSpeed, 
                       PULONG lpBytesPerFrame, PHANDLE lphBandwidth)
{
    clsIsochAllocateBandwidth ab;

    ab.nMaxBytesPerFrameRequested = *lpBytesPerFrame;
    ab.fulSpeed = *lpSpeed;
    ab.lpBandwidth = lphBandwidth;
    ab.lpBytesPerFrameAvailable = lpBytesPerFrame;
    ab.lpSpeedSelected = lpSpeed;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_ALLOCATE_BANDWIDTH,
                                  &ab,
                                  sizeof(clsIsochAllocateBandwidth),
                                  0,
                                  0,
                                  0,
                                  0));
}

/*==========================================================================*/
/*          Routine: AllocateResources                                      */
/*		                                                                    */
/*      Description: Allocates hardware/software resources for an           */
/*                   isochronous stream.                                    */
/*                                                                          */
/*==========================================================================*/
BOOL AllocateResources(cls1394HANDLE hDev, DWORD dwSpeed, DWORD dwFlags,
                       PHANDLE lphResources)
{
    clsIsochAllocateResources ar;

    ar.fulSpeed = dwSpeed;
    ar.fulFlags = dwFlags;
    ar.lpResources = lphResources;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_ALLOCATE_RESOURCES,
                                  &ar,
                                  sizeof(clsIsochAllocateResources),
                                  0,
                                  0,
                                  0,
                                  0));
}

/*==========================================================================*/
/*          Routine: AttachBuffers                                          */
/*		                                                                    */
/*      Description: Attaches Isochronous Buffers to an allocated resource  */
/*                                                                          */
/*==========================================================================*/
BOOL AttachBuffers(cls1394HANDLE hDev, HANDLE hResources, DWORD Channel,
                   PISOCH_DESCRIPTOR lpIsochDescriptor)
{
    clsIsochAttachBuffers ab;

    ab.hResources = hResources;
    ab.Channel = Channel;
    ab.lpIsochDescriptor = lpIsochDescriptor;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_ATTACH_BUFFERS,
                                  &ab,
                                  sizeof(clsIsochAttachBuffers),
                                  0,
                                  0,
                                  0,
                                  0));
}

/*==========================================================================*/
/*          Routine: IsochListen                                            */
/*		                                                                    */
/*      Description: Starts transferring isoch data to the allocated buffer */
/*                                                                          */
/*==========================================================================*/
BOOL IsochListen(cls1394HANDLE hDev, HANDLE hResources, DWORD Channel)
{
    clsIsochListen il;

    il.nChannel = Channel;
    il.hResources = hResources;
    il.fulFlags = 0;
    il.nStartCycle = 0;
    il.StartTime.HighPart = 0;
    il.StartTime.LowPart = 0;
    il.ulSynchronize = 0;
    il.ulTag = 0;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_ISOCH_LISTEN,
                                  &il,
                                  sizeof(clsIsochListen),
                                  0,
                                  0,
                                  0,
                                  0));
}

/*==========================================================================*/
/*          Routine: WriteQuadlet                                           */
/*		                                                                    */
/*      Description: Writes a quadlet to the 1394 device's memory space     */
/*                                                                          */
/*==========================================================================*/
BOOL WriteQuadlet(cls1394HANDLE hDev, LARGE_INTEGER liAddress, QUADLET qValue)
{
    clsAsyncWrite aw;
    BOOL rc;
    int i;

    aw.DestinationAddress = &liAddress;
    aw.nNumberOfBytesToWrite = sizeof(QUADLET);
    aw.nBlockSize = 0;
    aw.fulFlags = 0;
    aw.lpBuffer = &qValue;

    rc = cls1394DeviceIoControl(hDev,
                                CLS_REQUEST_ASYNC_WRITE,
                                &aw,
                                sizeof(clsAsyncWrite),
                                0,
                                0,
                                0,
                                0);

    for(i=0; i<20; i++)
    {
        ctMicrosec(1);
    }

    return(rc);
}

/*==========================================================================*/
/*          Routine: FreeChannel                                            */
/*		                                                                    */
/*      Description: Frees a previously allocated channel                   */
/*                                                                          */
/*==========================================================================*/
BOOL FreeChannel(cls1394HANDLE hDev, DWORD Channel)
{
    clsIsochFreeChannel fc;

    fc.nChannel = Channel;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_FREE_CHANNEL,
                                  &fc,
                                  sizeof(clsIsochFreeChannel),
                                  0,
                                  0,
                                  0,
                                  0));
}

/*==========================================================================*/
/*          Routine: FreeBandwidth                                          */
/*		                                                                    */
/*      Description: Frees previously allocated isochronous bandwidth       */
/*                                                                          */
/*==========================================================================*/
BOOL FreeBandwidth(cls1394HANDLE hDev, HANDLE hBandwidth)
{
    clsIsochFreeBandwidth fb;

    fb.hBandwidth = hBandwidth;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_FREE_BANDWIDTH,
                                  &fb,
                                  sizeof(clsIsochFreeBandwidth),
                                  0,
                                  0,
                                  0,
                                  0));
}

/*==========================================================================*/
/*          Routine: FreeResources                                          */
/*		                                                                    */
/*      Description: Frees hardware/software resources previously committed */
/*                   to an isochronous stream.                              */
/*                                                                          */
/*==========================================================================*/
BOOL FreeResources(cls1394HANDLE hDev, HANDLE hResources)
{
    clsIsochFreeResources fr;

    fr.hResources = hResources;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_FREE_RESOURCES,
                                  &fr,
                                  sizeof(clsIsochFreeResources),
                                  0,
                                  0,
                                  0,
                                  0));
}


/*==========================================================================*/
/*          Routine: DetachBuffers                                          */
/*		                                                                    */
/*      Description: Detaches Isochronous Buffers from an allocated resource*/
/*                                                                          */
/*==========================================================================*/
BOOL DetachBuffers(cls1394HANDLE hDev, HANDLE hResources,
                   PISOCH_DESCRIPTOR lpIsochDescriptor)
{
    clsIsochDetachBuffers db;

    db.hResources = hResources;
    db.lpIsochDescriptor = lpIsochDescriptor;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_DETACH_BUFFERS,
                                  &db,
                                  sizeof(clsIsochDetachBuffers),
                                  0,
                                  0,
                                  0,
                                  0));
}

/*==========================================================================*/
/*          Routine: IsochStop                                              */
/*		                                                                    */
/*      Description: Stops transferring isoch data to the allocated buffer  */
/*                                                                          */
/*==========================================================================*/
BOOL IsochStop(cls1394HANDLE hDev, HANDLE hResources, DWORD Channel)
{
    clsIsochStop is;

    is.nChannel = Channel;
    is.hResources = hResources;
    is.fulFlags = 0;
    is.nStopCycle = 0;
    is.StopTime.HighPart = 0;
    is.StopTime.LowPart = 0;
    is.ulSynchronize = 0;
    is.ulTag = 0;

    return(cls1394DeviceIoControl(hDev,
                                  CLS_REQUEST_ISOCH_STOP,
                                  &is,
                                  sizeof(clsIsochStop),
                                  0,
                                  0,
                                  0,
                                  0));
}


/*==========================================================================*/
/*          Routine: Yuv2DIB                                                */
/*		                                                                    */
/*      Description: Converts YUV data to a device independent bitmap       */
/*                                                                          */
/*==========================================================================*/
#define LIMIT(x) (((x) < 0) ? 0 : ((x) > 255) ? 255 : (x))

PCHAR Yuv2DIB(char *YuvData)
{
    int i, j;
    BYTE y0, y1;
    //BYTE byu, byv;
    char u, v;
    BYTE *OrigDibData, *DibData;
    int r,g,b;
    BMPHEAD BmpHead;
    BMPHEAD *lpBmpHead = &BmpHead;

    BmpHead.bfType[0]       = 'B';
    BmpHead.bfType[1]       = 'M';
    BmpHead.bfSize          = ((320*240*3)+54);
    BmpHead.bfReserved[0]   = 0;
    BmpHead.bfReserved[1]   = 0;
    BmpHead.bfOffBits       = 54;
    BmpHead.biSize          = 40;
    BmpHead.biWidth         = 320;
    BmpHead.biHeight        = 240;
    BmpHead.biPlanes        = 1;
    BmpHead.biBitCount      = 24;
    BmpHead.biCompression   = 0;
    BmpHead.biSizeImage     = (320*240*3);
    BmpHead.biXPelsPerMeter = 0;
    BmpHead.biYPelsPerMeter = 0;
    BmpHead.biClrUsed       = 0;
    BmpHead.biClrImportant  = 0;

    DibData = GlobalAlloc(GMEM_FIXED | GMEM_ZEROINIT,((320*240*3)+54));

    for(i=0; i<54; i++)
    {
        DibData[i] = ((BYTE *)lpBmpHead)[i];
    }

    OrigDibData = DibData+54;

    DibData += ((960*239)+54);     //point to last line in DIB buffer

    for(j = 0; j < 240; j++)
    {
        for(i = 0; i < 160; i++)
        {
            y1 = *YuvData++;
            v  = *YuvData++;
            y0 = *YuvData++;
            u  = *YuvData++;

            u -= ((char)128);
            v -= ((char)128);

            r = (int)(y0 + (1.14 * v));
            g = (int)(y0 - ((0.395 * u) - (0.581 * v)));
            b = (int)(y0 + (2.032 * u));

            *DibData = (BYTE)(LIMIT(b));
            ++DibData;
            *DibData = (BYTE)(LIMIT(g));
            ++DibData;
            *DibData = (BYTE)(LIMIT(r));
            ++DibData;
        
            r = (int)(y1 + (1.14 * v));
            g = (int)(y1 - ((0.395 * u) - (0.581 * v)));
            b = (int)(y1 + (2.032 * u));

            *DibData = (BYTE)(LIMIT(b));
            ++DibData;
            *DibData = (BYTE)(LIMIT(g));
            ++DibData;
            *DibData = (BYTE)(LIMIT(r));
            ++DibData;
        }    
        if((DibData - 960) != OrigDibData)
        {
            DibData -= 1920;
        }
    }    

    return(OrigDibData-54);
}
